@@ -300,6 +300,12 @@ class ABCActorMixin(ABCActorIdentity, ABCTick, ABCTimerQueue):
         Returns:
             query response
         """
+    @abstractmethod
+    def execute_on_actor_thread(self, *, runnable: ABCActorRunnable):
+        """
+        Execute on Actor Thread and Wait until response is processed
+        @params runnable: reservation to be processed
+        """
 
     @abstractmethod
     def execute_on_actor_thread_and_wait(self, *, runnable: ABCActorRunnable):
@@ -420,6 +426,17 @@ class ABCActorMixin(ABCActorIdentity, ABCTick, ABCTimerQueue):
             Exception in case of error
         """
 
+    @abstractmethod
+    def modify_slice(self, *, slice_object: ABCSlice):
+        """
+        Modify the slice registered with the actor. Moves the slice into Modifying State
+
+        Args:
+            slice_object: slice_object
+        Raises:
+            Exception in case of error
+        """
+
     @abstractmethod
     def remove_slice(self, *, slice_object: ABCSlice):
         """
@@ -442,6 +459,17 @@ class ABCActorMixin(ABCActorIdentity, ABCTick, ABCTimerQueue):
             Exception in case of error
         """
 
+    @abstractmethod
+    def modify_accept(self, *, slice_id: ID):
+        """
+        Accept the last modify
+
+        Args:
+            slice_id: slice id
+        Raises:
+            Exception in case of error
+        """
+
     @abstractmethod
     def re_register_slice(self, *, slice_object: ABCSlice):
         """
@@ -523,6 +551,19 @@ class ABCActorMixin(ABCActorIdentity, ABCTick, ABCTimerQueue):
             Exception in case of error
         """
 
+    @abstractmethod
+    def close_delegation(self, *, did: str):
+        """
+        Closes the delegation. Note: the delegation must have already been registered with the actor.
+        This method may involve either a client or a server side action or both. When called on a broker,
+        this method will only close the broker delegation.
+
+        Args:
+            did: delegation id
+        Raises:
+            Exception in case of error
+        """
+
     @abstractmethod
     def close(self, *, reservation: ABCReservationMixin):
         """
@@ -660,3 +701,19 @@ class ABCActorMixin(ABCActorIdentity, ABCTick, ABCTimerQueue):
         Raises:
             Exception in case of error
         """
+
+    def get_asm_thread(self):
+        return None
+
+    @abstractmethod
+    def remove_delegation(self, *, did: str):
+        """
+        Removes the specified delegation. Note: the delegation must have already been registered with the actor.
+        This method will unregister the reservation and remove it from the underlying database.
+        Only closed and failed delegation can be removed.
+
+        Args:
+            did: delegation id
+        Raises:
+            Exception if an error occurs or when trying to remove a delegation that is neither failed or closed.
+        """
\ No newline at end of file
