@@ -25,6 +25,7 @@
 # Author: Komal Thareja (kthare10@renci.org)
 import queue
 import threading
+import time
 import traceback
 from typing import List
 
@@ -40,12 +41,14 @@ from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
 from fabric_cf.actor.core.apis.abc_slice import ABCSlice
 from fabric_cf.actor.core.common.exceptions import ActorException
 from fabric_cf.actor.core.container.message_service import MessageService
+from fabric_cf.actor.core.core.event_processor import TickEvent, EventType, EventProcessor
 from fabric_cf.actor.core.delegation.delegation_factory import DelegationFactory
 from fabric_cf.actor.core.kernel.failed_rpc import FailedRPC
 from fabric_cf.actor.core.kernel.kernel_wrapper import KernelWrapper
 from fabric_cf.actor.core.kernel.rpc_manager_singleton import RPCManagerSingleton
 from fabric_cf.actor.core.kernel.resource_set import ResourceSet
 from fabric_cf.actor.core.kernel.slice import SliceTypes
+from fabric_cf.actor.core.kernel.slice_state_machine import SliceState
 from fabric_cf.actor.core.proxies.proxy import Proxy
 from fabric_cf.actor.core.time.actor_clock import ActorClock
 from fabric_cf.actor.core.time.term import Term
@@ -56,53 +59,12 @@ from fabric_cf.actor.core.util.reservation_set import ReservationSet
 from fabric_cf.actor.security.auth_token import AuthToken
 
 
-class ExecutionStatus:
-    """
-    Execution status of an action on Actor Thread
-    """
-    def __init__(self):
-        self.done = False
-        self.exception = None
-        self.result = None
-        self.lock = threading.Condition()
-
-    def mark_done(self):
-        """
-        Mark as done
-        """
-        self.done = True
-
-
-class ActorEvent(ABCActorEvent):
-    """
-    Actor Event
-    """
-    def __init__(self, *, status: ExecutionStatus, runnable: ABCActorRunnable):
-        self.status = status
-        self.runnable = runnable
-
-    def process(self):
-        """
-        Process an event
-        """
-        try:
-            self.status.result = self.runnable.run()
-        except Exception as e:
-            traceback.print_exc()
-            self.status.exception = e
-        finally:
-            with self.status.lock:
-                self.status.done = True
-                self.status.lock.notify_all()
-
-
 class ActorMixin(ABCActorMixin):
     """
     Actor is the base class for all actor implementations
     """
     DefaultDescription = "no description"
-
-    actor_count = 0
+    SUPPORTED_EVENTS = [EventType.TickEvent, EventType.InterActorEvent, EventType.SyncEvent]
 
     def __init__(self, *, auth: AuthToken = None, clock: ActorClock = None):
         # Globally unique identifier for this actor.
@@ -135,19 +97,11 @@ class ActorMixin(ABCActorMixin):
         self.stopped = False
         # Initialization status.
         self.initialized = False
-        # Contains a reference to the thread currently executing the timer handler.
-        # This field is set at the entry to and clear at the exit.
-        # The primary use of the field is to handle correctly stopping the actor.
-        self.thread = None
-        # A queue of timers that have fired and need to be processed.
-        self.timer_queue = queue.Queue()
-        self.event_queue = queue.Queue()
         # Reservations to close once recovery is complete.
         self.closing = ReservationSet()
 
-        self.thread_lock = threading.Lock()
-        self.actor_main_lock = threading.Condition()
         self.message_service = None
+        self.event_processors = {}
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -159,13 +113,9 @@ class ActorMixin(ABCActorMixin):
         del state['first_tick']
         del state['stopped']
         del state['initialized']
-        del state['thread_lock']
-        del state['thread']
-        del state['timer_queue']
-        del state['event_queue']
-        del state['actor_main_lock']
         del state['closing']
         del state['message_service']
+        del state['event_processors']
         return state
 
     def __setstate__(self, state):
@@ -178,14 +128,10 @@ class ActorMixin(ABCActorMixin):
         self.first_tick = True
         self.stopped = False
         self.initialized = False
-        self.thread = None
-        self.thread_lock = threading.Lock()
-        self.timer_queue = queue.Queue()
-        self.event_queue = queue.Queue()
-        self.actor_main_lock = threading.Condition()
         self.closing = ReservationSet()
         self.message_service = None
         self.policy.set_actor(actor=self)
+        self.event_processors = {}
 
     def actor_added(self):
         self.plugin.actor_added()
@@ -199,6 +145,9 @@ class ActorMixin(ABCActorMixin):
     def fail_delegation(self, *, did: str, message: str):
         self.wrapper.fail_delegation(did=did, message=message)
 
+    def close_delegation(self, *, did: str):
+        self.wrapper.close_delegation(did=did)
+
     def close_by_rid(self, *, rid: ID):
         self.wrapper.close(rid=rid)
 
@@ -240,19 +189,7 @@ class ActorMixin(ABCActorMixin):
 
     def external_tick(self, *, cycle: int):
         self.logger.info("External Tick start cycle: {}".format(cycle))
-
-        class TickEvent(ABCActorEvent):
-            def __init__(self, *, base, cycle: int):
-                self.base = base
-                self.cycle = cycle
-
-            def __str__(self):
-                return "{} {}".format(self.base, self.cycle)
-
-            def process(self):
-                self.base.actor_tick(cycle=self.cycle)
-
-        self.queue_event(incoming=TickEvent(base=self, cycle=cycle))
+        self.queue_event(incoming=TickEvent(actor=self, cycle=cycle))
         self.logger.info("External Tick end cycle: {}".format(cycle))
 
     def actor_tick(self, *, cycle: int):
@@ -265,7 +202,6 @@ class ActorMixin(ABCActorMixin):
             if not self.recovered:
                 self.logger.warning("Tick for an actor that has not completed recovery")
                 return
-            current_cycle = 0
             if self.first_tick:
                 current_cycle = cycle
             else:
@@ -303,7 +239,7 @@ class ActorMixin(ABCActorMixin):
     def get_description(self) -> str:
         return self.description
 
-    def get_guid(self) -> ID:
+    def get_guid(self) -> ID or None:
         if self.identity is not None:
             return self.identity.get_guid()
         return None
@@ -383,6 +319,8 @@ class ActorMixin(ABCActorMixin):
 
             self.current_cycle = -1
 
+            for x in self.SUPPORTED_EVENTS:
+                self.event_processors[x] = EventProcessor(name=str(x), logger=self.logger)
             self.setup_message_service()
 
             self.initialized = True
@@ -394,7 +332,7 @@ class ActorMixin(ABCActorMixin):
         return self.stopped
 
     def query(self, *, query: dict = None, caller: AuthToken = None, actor_proxy: ABCActorProxy = None,
-              handler: ABCQueryResponseHandler = None) -> dict:
+              handler: ABCQueryResponseHandler = None) -> dict or None:
         """
         Query an actor
         @param query query
@@ -425,7 +363,14 @@ class ActorMixin(ABCActorMixin):
 
         self.logger.debug("Recovering client slices")
         client_slices = self.plugin.get_database().get_slices(slc_type=[SliceTypes.ClientSlice,
-                                                                        SliceTypes.BrokerClientSlice])
+                                                                        SliceTypes.BrokerClientSlice],
+                                                              state=[SliceState.Configuring.value,
+                                                                     SliceState.Nascent.value,
+                                                                     SliceState.StableOK.value,
+                                                                     SliceState.StableError.value,
+                                                                     SliceState.Modifying.value,
+                                                                     SliceState.ModifyOK.value,
+                                                                     SliceState.ModifyError.value])
         self.logger.debug("Found {} client slices".format(len(client_slices)))
         self.recover_slices(slices=client_slices)
         self.logger.debug("Recovery of client slices complete")
@@ -503,7 +448,8 @@ class ActorMixin(ABCActorMixin):
 
         adms = self.policy.aggregate_resource_model.generate_adms()
 
-        # Create new delegations and add to the broker slice; they will be re-registered with the policy in the recovery
+        # Create new delegations and add to the broker slice;
+        # they will be re-registered with the policy in the recovery
         for name in delegation_names:
             new_delegation_graph = adms.get(name)
             dlg_obj = DelegationFactory.create(did=new_delegation_graph.get_graph_id(),
@@ -698,9 +644,15 @@ class ActorMixin(ABCActorMixin):
     def register_slice(self, *, slice_object: ABCSlice):
         self.wrapper.register_slice(slice_object=slice_object)
 
+    def modify_slice(self, *, slice_object: ABCSlice):
+        self.wrapper.modify_slice(slice_object=slice_object)
+
     def register_delegation(self, *, delegation: ABCDelegation):
         self.wrapper.register_delegation(delegation=delegation)
 
+    def remove_delegation(self, *, did: str):
+        self.wrapper.remove_delegation(did=did)
+
     def remove_reservation(self, *, reservation: ABCReservationMixin = None, rid: ID = None):
         if reservation is not None:
             self.wrapper.remove_reservation(rid=reservation.get_reservation_id())
@@ -714,6 +666,9 @@ class ActorMixin(ABCActorMixin):
     def remove_slice_by_slice_id(self, *, slice_id: ID):
         self.wrapper.remove_slice(slice_id=slice_id)
 
+    def modify_accept(self, *, slice_id: ID):
+        self.wrapper.modify_accept(slice_id=slice_id)
+
     def re_register_delegation(self, *, delegation: ABCDelegation):
         self.wrapper.re_register_delegation(delegation=delegation)
 
@@ -789,73 +744,27 @@ class ActorMixin(ABCActorMixin):
         """
         self.stopped = value
 
-    def is_on_actor_thread(self) -> bool:
+    def execute_on_actor_thread(self, *, runnable: ABCActorRunnable):
         """
-        Check if running on actor thread
-        @return true if running on actor thread, false otherwise
+        Execute an incoming action on actor thread
+        @param runnable incoming action/operation
         """
-        result = False
-        try:
-            self.thread_lock.acquire()
-            result = self.thread == threading.current_thread()
-        finally:
-            self.thread_lock.release()
-        return result
+        self.event_processors[EventType.InterActorEvent].execute_on_thread_async(runnable=runnable)
 
     def execute_on_actor_thread_and_wait(self, *, runnable: ABCActorRunnable):
         """
         Execute an incoming action on actor thread
         @param runnable incoming action/operation
         """
-        if self.is_on_actor_thread():
-            return runnable.run()
-        else:
-            status = ExecutionStatus()
-            event = ActorEvent(status=status, runnable=runnable)
-
-            self.queue_event(incoming=event)
-
-            with status.lock:
-                while not status.done:
-                    status.lock.wait()
-
-            if status.exception is not None:
-                raise status.exception
-
-            return status.result
-
-    def run(self):
-        """
-        Actor run function for actor thread
-        """
-        try:
-            self.logger.info("Actor Main Thread started")
-            self.actor_count -= 1
-            self.actor_main()
-        except Exception as e:
-            self.logger.error(f"Unexpected error {e}")
-            self.logger.error(traceback.format_exc())
-        finally:
-            self.logger.info("Actor Main Thread exited")
+        return self.event_processors[EventType.SyncEvent].execute_on_thread_sync(runnable=runnable)
 
     def start(self):
         """
         Start an Actor
         """
-        try:
-            self.thread_lock.acquire()
-            if self.thread is not None:
-                raise ActorException("This actor has already been started")
-
-            self.thread = threading.Thread(target=self.run)
-            self.thread.setName(self.get_name())
-            self.thread.setDaemon(True)
-            self.thread.start()
-        finally:
-            self.thread_lock.release()
-
+        for x in self.event_processors.values():
+            x.start()
         self.message_service.start()
-
         if self.plugin.get_handler_processor() is not None:
             self.plugin.get_handler_processor().start()
 
@@ -865,25 +774,9 @@ class ActorMixin(ABCActorMixin):
         """
         self.stopped = True
         self.message_service.stop()
-        try:
-            self.thread_lock.acquire()
-            temp = self.thread
-            self.thread = None
-            if temp is not None:
-                self.logger.warning("It seems that the actor thread is running. Interrupting it")
-                try:
-                    # TODO find equivalent of interrupt
-                    with self.actor_main_lock:
-                        self.actor_main_lock.notify_all()
-                    temp.join()
-                except Exception as e:
-                    self.logger.error("Could not join actor thread {}".format(e))
-                    self.logger.error(traceback.format_exc())
-                finally:
-                    self.thread_lock.release()
-        finally:
-            if self.thread_lock is not None and self.thread_lock.locked():
-                self.thread_lock.release()
+
+        for x in self.event_processors.values():
+            x.start()
 
         if self.plugin.get_handler_processor() is not None:
             self.plugin.get_handler_processor().shutdown()
@@ -932,19 +825,34 @@ class ActorMixin(ABCActorMixin):
         """
         Queue an event on Actor timer queue
         """
-        with self.actor_main_lock:
-            self.timer_queue.put_nowait(timer)
+        try:
+            self.event_processors[EventType.InterActorEvent].enqueue(incoming=timer)
             self.logger.debug("Added timer to timer queue {}".format(timer.__class__.__name__))
-            self.actor_main_lock.notify_all()
+        except Exception as e:
+            self.logger.error(f"Failed to queue event: {timer.__class__.__name__} e: {e}")
 
     def queue_event(self, *, incoming: ABCActorEvent):
         """
         Queue an even on Actor Event Queue
         """
-        with self.actor_main_lock:
-            self.event_queue.put_nowait(incoming)
+        try:
+            if isinstance(incoming, TickEvent):
+                self.event_processors[EventType.TickEvent].enqueue(incoming=incoming)
+            else:
+                self.event_processors[EventType.InterActorEvent].enqueue(incoming=incoming)
             self.logger.debug("Added event to event queue {}".format(incoming.__class__.__name__))
-            self.actor_main_lock.notify_all()
+        except Exception as e:
+            self.logger.error(f"Failed to queue event: {incoming.__class__.__name__} e: {e}")
+
+    def queue_event_sync(self, *, incoming: ABCActorEvent):
+        """
+        Queue an even on Actor Event Queue
+        """
+        try:
+            self.event_processors[EventType.SyncEvent].enqueue(incoming=incoming)
+            self.logger.debug("Added event to event queue {}".format(incoming.__class__.__name__))
+        except Exception as e:
+            self.logger.error(f"Failed to queue event: {incoming.__class__.__name__} e: {e}")
 
     def await_no_pending_reservations(self):
         """
@@ -952,66 +860,6 @@ class ActorMixin(ABCActorMixin):
         """
         self.wrapper.await_nothing_pending()
 
-    def actor_main(self):
-        """
-        Actor Main loop
-        """
-        while True:
-            events = []
-            timers = []
-
-            with self.actor_main_lock:
-
-                while self.event_queue.empty() and self.timer_queue.empty() and not self.stopped:
-                    try:
-                        self.actor_main_lock.wait()
-                    except InterruptedError as e:
-                        self.logger.info("Actor thread interrupted. Exiting")
-                        return
-
-                if self.stopped:
-                    self.logger.info("Actor exiting")
-                    return
-
-                if not self.event_queue.empty():
-                    try:
-                        for event in IterableQueue(source_queue=self.event_queue):
-                            events.append(event)
-                    except Exception as e:
-                        self.logger.error(f"Error while adding event to event queue! e: {e}")
-                        self.logger.error(traceback.format_exc())
-
-                if not self.timer_queue.empty():
-                    try:
-                        for timer in IterableQueue(source_queue=self.timer_queue):
-                            timers.append(timer)
-                    except Exception as e:
-                        self.logger.error(f"Error while adding event to event queue! e: {e}")
-                        self.logger.error(traceback.format_exc())
-
-                self.actor_main_lock.notify_all()
-
-            if len(events) > 0:
-                self.logger.debug(f"Processing {len(events)} events")
-                for event in events:
-                    #self.logger.debug("Processing event of type {}".format(type(event)))
-                    #self.logger.debug("Processing event {}".format(event))
-                    try:
-                        event.process()
-                        #self.logger.debug("Processing event {} done".format(event))
-                    except Exception as e:
-                        self.logger.error(f"Error while processing event {type(event)}, {e}")
-                        self.logger.error(traceback.format_exc())
-
-            if len(timers) > 0:
-                self.logger.debug(f"Processing {len(timers)} timers")
-                for t in timers:
-                    try:
-                        t.execute()
-                    except Exception as e:
-                        self.logger.error(f"Error while processing a timer {type(t)}, {e}")
-                        self.logger.error(traceback.format_exc())
-
     def setup_message_service(self):
         """
         Set up Message Service for incoming Kafka Messages
