@@ -23,6 +23,7 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
+import threading
 import traceback
 
 from fabric_cf.actor.core.apis.abc_broker_proxy import ABCBrokerProxy
@@ -39,13 +40,11 @@ from fabric_cf.actor.core.util.update_data import UpdateData
 
 
 class BrokerDelegation(Delegation):
-    def __init__(self, dlg_graph_id: str, slice_id: ID, broker: ABCBrokerProxy = None):
-        super().__init__(dlg_graph_id=dlg_graph_id, slice_id=slice_id)
+    def __init__(self, dlg_graph_id: str, slice_id: ID, broker: ABCBrokerProxy = None, delegation_name: str = None):
+        super().__init__(dlg_graph_id=dlg_graph_id, slice_id=slice_id, delegation_name=delegation_name)
         self.exported = False
         self.broker = broker
         self.authority = None
-        # Relinquish status.
-        self.relinquished = False
         # The status of the last ticket update.
         self.last_delegation_update = UpdateData()
 
@@ -56,6 +55,7 @@ class BrokerDelegation(Delegation):
         del state['slice_object']
         del state['logger']
         del state['policy']
+        del state['thread_lock']
         return state
 
     def __setstate__(self, state):
@@ -65,6 +65,7 @@ class BrokerDelegation(Delegation):
         self.slice_object = None
         self.logger = None
         self.policy = None
+        self.thread_lock = threading.Lock()
 
     def get_broker(self) -> ABCBrokerProxy:
         """
@@ -90,13 +91,13 @@ class BrokerDelegation(Delegation):
     def claim(self):
         raise DelegationException("Not supported on Broker Delegation")
 
-    def delegate(self, policy: ABCPolicy, id_token: str = None):
+    def delegate(self, policy: ABCPolicy):
         self.policy = policy
 
         if self.state == DelegationState.Nascent or self.state == DelegationState.Reclaimed:
             if self.exported:
                 self.sequence_out += 1
-                RPCManagerSingleton.get().claim_delegation(delegation=self, id_token=id_token)
+                RPCManagerSingleton.get().claim_delegation(delegation=self)
                 self.transition(prefix="delegate", state=DelegationState.Delegated)
             else:
                 self.logger.error(
@@ -114,23 +115,22 @@ class BrokerDelegation(Delegation):
                 self.error_string_prefix.format(self, "initiating delegate on defunct Delegation"))
             raise DelegationException("initiating delegate on defunct Delegation")
 
-    def reclaim(self, id_token: str = None):
+    def reclaim(self):
         if self.state == DelegationState.Delegated:
             # We are an agent asked to return a pre-reserved "will call" ticket
             # to a client. Set mustSendUpdate so that the update will be sent
             # on the next probe.
             self.transition(prefix="reclaimed", state=DelegationState.Reclaimed)
             self.policy.update_delegation_complete(delegation=self, reclaim=True)
-            self.service_reclaim(id_token=id_token)
+            self.service_reclaim()
         else:
             self.logger.error(
                 self.error_string_prefix.format(self, self.invalid_state_prefix.format('reclaim', 'reclaim')))
             raise DelegationException(self.invalid_state_prefix.format('reclaim', 'reclaim'))
 
-    def service_reclaim(self, id_token: str = None):
+    def service_reclaim(self):
         """
         Service reclaim
-        @param id_token identity token
         """
         self.logger.debug("Servicing reclaim")
         if self.callback is None:
@@ -141,7 +141,7 @@ class BrokerDelegation(Delegation):
         try:
             if self.exported:
                 self.sequence_out += 1
-                RPCManagerSingleton.get().reclaim_delegation(delegation=self, id_token=id_token)
+                RPCManagerSingleton.get().reclaim_delegation(delegation=self)
         except Exception as e:
             # Note that this may result in a "stuck" delegation... not much we
             # can do if the receiver has failed or rejects our update. We will
@@ -160,35 +160,6 @@ class BrokerDelegation(Delegation):
             self.logger.error(self.error_string_prefix.format(self, Constants.NOT_SPECIFIED_PREFIX.format("graph id")))
             raise DelegationException(Constants.NOT_SPECIFIED_PREFIX.format("graph id"))
 
-    def do_relinquish(self):
-        """
-        Perform required actions for a relinquish
-        """
-        if not self.relinquished:
-            self.relinquished = True
-            try:
-                if self.policy is not None:
-                    self.policy.closed_delegation(delegation=self)
-                else:
-                    self.logger.warning("policy not set in reservation {}, unable to call policy.closed(), "
-                                        "continuing".format(self.dlg_graph_id))
-            except Exception as e:
-                self.logger.error("close with policy {}".format(e))
-
-            try:
-                self.sequence_out += 1
-                RPCManagerSingleton.get().relinquish_delegation(delegation=self)
-            except Exception as e:
-                self.logger.error("broker reports relinquish error: {}".format(e))
-
-    def close(self):
-        """
-        Close a delegation, remove the corresponding graph from CBM
-        """
-        if self.state == DelegationState.Nascent:
-            self.transition(prefix="close", state=DelegationState.Closed)
-            self.do_relinquish()
-
     def get_client_callback_proxy(self) -> ABCClientCallbackProxy:
         """
         Get Client callback proxy
@@ -220,6 +191,7 @@ class BrokerDelegation(Delegation):
         if self.authority is None and incoming.get_site_proxy() is not None:
             self.authority = incoming.get_site_proxy()
 
+        self.delegation_name = incoming.get_delegation_name()
         self.graph = incoming.get_graph()
         self.policy.update_delegation_complete(delegation=self)
         if self.graph is not None:
