@@ -25,11 +25,11 @@
 # Author: Komal Thareja (kthare10@renci.org)
 from __future__ import annotations
 
+import threading
 from typing import TYPE_CHECKING
 
-from datetime import datetime
+from datetime import datetime, timezone
 from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin, ReservationCategory
-from fabric_cf.actor.core.apis.abc_kernel_reservation import ABCKernelReservation
 from fabric_cf.actor.core.common.exceptions import ReservationException
 from fabric_cf.actor.core.kernel.reservation_states import ReservationStates, ReservationPendingStates, JoinState
 from fabric_cf.actor.core.util.reservation_state import ReservationState
@@ -38,7 +38,7 @@ if TYPE_CHECKING:
     from fabric_cf.actor.core.apis.abc_actor_mixin import ABCActorMixin
     from fabric_cf.actor.core.apis.abc_policy import ABCPolicy
     from fabric_cf.actor.core.apis.abc_slice import ABCSlice
-    from fabric_cf.actor.core.apis.abc_kernel_slice import ABCKernelSlice
+    from fabric_cf.actor.core.apis.abc_slice import ABCSlice
     from fabric_cf.actor.core.kernel.request_types import RequestTypes
     from fabric_cf.actor.core.kernel.resource_set import ResourceSet
     from fabric_cf.actor.core.time.term import Term
@@ -46,7 +46,7 @@ if TYPE_CHECKING:
     from fabric_cf.actor.core.util.resource_type import ResourceType
 
 
-class Reservation(ABCKernelReservation):
+class Reservation(ABCReservationMixin):
     """
     These are the only methods synchronized on the Reservation object itself. The
     purpose is to allow an external thread to await a state transition in a
@@ -68,7 +68,7 @@ class Reservation(ABCKernelReservation):
     """
 
     def __init__(self, *, rid: ID = None, resources: ResourceSet = None, term: Term = None,
-                 slice_object: ABCKernelSlice = None):
+                 slice_object: ABCSlice = None):
         # The unique reservation identifier.
         self.rid = rid
         # Reservation category. Subclasses should supply the correct value.
@@ -138,6 +138,7 @@ class Reservation(ABCKernelReservation):
         self.service_pending = ReservationPendingStates.None_
         self.last_transition_time = None
         self.last_pending_state = ReservationPendingStates.None_
+        self.thread_lock = threading.Lock()
 
     def __getstate__(self):
         state = self.__dict__.copy()
@@ -152,6 +153,7 @@ class Reservation(ABCKernelReservation):
         del state['pending_recover']
         del state['state_transition']
         del state['service_pending']
+        del state['thread_lock']
         return state
 
     def __setstate__(self, state):
@@ -167,6 +169,7 @@ class Reservation(ABCKernelReservation):
         self.pending_recover = False
         self.state_transition = False
         self.service_pending = ReservationPendingStates.None_
+        self.thread_lock = threading.Lock()
 
     def restore(self, *, actor: ABCActorMixin, slice_obj: ABCSlice):
         """
@@ -288,7 +291,7 @@ class Reservation(ABCKernelReservation):
     def get_category(self) -> ReservationCategory:
         return self.category
 
-    def get_kernel_slice(self) -> ABCKernelSlice:
+    def get_kernel_slice(self) -> ABCSlice:
         return self.slice
 
     def get_leased_abstract_units(self) -> int:
@@ -608,7 +611,7 @@ class Reservation(ABCKernelReservation):
 
         self.set_dirty()
         self.state_transition = True
-        self.last_transition_time = datetime.utcnow()
+        self.last_transition_time = datetime.now(timezone.utc)
 
     def update_lease(self, *, incoming: ABCReservationMixin, update_data):
         self.internal_error(err="abstract update_lease trap")
@@ -650,15 +653,6 @@ class Reservation(ABCKernelReservation):
         """
         return JoinState.None_
 
-    class CountHelper:
-        """
-        Helper class for counting units.
-        """
-        def __init__(self):
-            self.pending = 0
-            self.active = 0
-            self.type = None
-
     def get_graph_node_id(self) -> str:
         if self.requested_resources is not None:
             request = self.requested_resources.get_sliver()
@@ -677,12 +671,18 @@ class Reservation(ABCKernelReservation):
         if self.last_transition_time is None:
             return False
 
-        current_time = datetime.utcnow()
+        current_time = datetime.now(timezone.utc)
         if (current_time - self.last_transition_time).seconds > timeout:
-            print("Return True")
             return True
 
         return False
 
     def get_error_message(self) -> str:
         return self.error_message
+
+    def lock(self):
+        self.thread_lock.acquire()
+
+    def unlock(self):
+        if self.thread_lock.locked():
+            self.thread_lock.release()
\ No newline at end of file
