@@ -26,11 +26,11 @@
 from __future__ import annotations
 
 import json
+import threading
+import time
 import traceback
 from typing import TYPE_CHECKING, List
 
-from datetime import datetime
-
 from fim.slivers.attached_components import ComponentType
 from fim.slivers.base_sliver import BaseSliver
 from fim.slivers.capacities_labels import Labels
@@ -65,7 +65,6 @@ if TYPE_CHECKING:
     from fabric_cf.actor.core.apis.abc_slice import ABCSlice
     from fabric_cf.actor.core.kernel.resource_set import ResourceSet
     from fabric_cf.actor.core.time.term import Term
-    from fabric_cf.actor.core.util.resource_count import ResourceCount
     from fabric_cf.actor.core.util.resource_type import ResourceType
 
 
@@ -202,6 +201,7 @@ class ReservationClient(Reservation, ABCControllerReservation):
         del state['service_pending']
 
         del state['suggested']
+        del state['thread_lock']
         return state
 
     def __setstate__(self, state):
@@ -219,6 +219,7 @@ class ReservationClient(Reservation, ABCControllerReservation):
         self.service_pending = ReservationPendingStates.None_
 
         self.suggested = True
+        self.thread_lock = threading.Lock()
 
     def restore(self, *, actor: ABCActorMixin, slice_obj: ABCSlice):
         """
@@ -316,8 +317,10 @@ class ReservationClient(Reservation, ABCControllerReservation):
         # Alternative: could transition to (state, None) to allow retry of the
         # redeem/extend by a higher level.
         if update_data.failed:
-            self.transition(prefix="failed lease update", state=ReservationStates.Failed,
-                            pending=ReservationPendingStates.None_)
+            self.fail(message=f"failed lease update- {update_data.get_message()}",
+                      sliver=incoming.get_resources().get_sliver())
+            #self.transition(prefix="failed lease update", state=ReservationStates.Failed,
+            #                pending=ReservationPendingStates.None_)
         else:
             try:
                 self.lease_update_satisfies(incoming=incoming, update_data=update_data)
@@ -418,7 +421,7 @@ class ReservationClient(Reservation, ABCControllerReservation):
                                   " ignoring it: {}".format(pred_state.get_reservation()))
                 continue
 
-            if not pred_state.get_reservation().is_active_joined():
+            if not pred_state.get_reservation().is_active():
                 approved = False
                 break
 
@@ -428,9 +431,10 @@ class ReservationClient(Reservation, ABCControllerReservation):
         return approved
 
     def can_redeem(self) -> bool:
-        if (((self.state == ReservationStates.ActiveTicketed) or (self.state == ReservationStates.Ticketed) or
-             ((self.state == ReservationStates.Active) and self.pending_recover)) and
-                (self.pending_state == ReservationPendingStates.None_)):
+        ticketed_states_to_redeem = [ReservationStates.ActiveTicketed, ReservationStates.Ticketed]
+        is_valid_state = self.state in ticketed_states_to_redeem or \
+                         (self.state == ReservationStates.Active and self.pending_recover)
+        if is_valid_state and self.pending_state == ReservationPendingStates.None_:
             assert self.resources is not None
             c = self.resources.get_resources()
             assert c is not None and c.get_units() > 0
@@ -461,8 +465,7 @@ class ReservationClient(Reservation, ABCControllerReservation):
                 self.logger.error(f"Redeem predecessors not found {rid} for {self.get_reservation_id()}")
                 continue
             parent_res = pred_state.get_reservation()
-            if parent_res is not None and \
-                    ReservationStates(parent_res.get_state()) == ReservationStates.Ticketed:
+            if parent_res is not None and (parent_res.is_ticketed() or parent_res.is_active()):
                 node_sliver = parent_res.get_resources().get_sliver()
                 component = node_sliver.attached_components_info.get_device(name=component_name)
                 graph_id, bqm_component_id = component.get_node_map()
@@ -490,7 +493,6 @@ class ReservationClient(Reservation, ABCControllerReservation):
         facilitate ticketing from the broker. Note that approve_ticket may be polled multiple
         times, and should be idempotent.
 
-
         @return true if approved; false otherwise
         """
         approved = True
@@ -505,7 +507,7 @@ class ReservationClient(Reservation, ABCControllerReservation):
                 self.fail(message=f"redeem predecessor reservation# {pred_state.get_reservation().get_reservation_id()}"
                                   f" is in a terminal state")
 
-            if not (pred_state.get_reservation().is_ticketed() or pred_state.get_reservation().is_active_ticketed()):
+            if not (pred_state.get_reservation().is_ticketed() or pred_state.get_reservation().is_active()):
                 approved = False
                 break
 
@@ -620,36 +622,6 @@ class ReservationClient(Reservation, ABCControllerReservation):
                                               pending=ReservationPendingStates.Closing,
                                               join_state=JoinState.NoJoin)
 
-    def count_with_time(self, *, when: datetime):
-        """
-        Count with time
-        @param when ime
-        """
-        result = Reservation.CountHelper()
-        if not self.is_terminal():
-            if self.term is not None and self.term.contains(term=when) and self.resources is not None:
-                result.active = self.resources.get_concrete_units(when=when)
-                result.type = self.resources.type
-            else:
-                if self.approved and self.approved_term is not None and self.approved_term.contains(term=when) and \
-                        self.approved_resources is not None:
-                    result.pending = self.approved_resources.units
-                    result.type = self.approved_resources.type
-        return result
-
-    def count(self, *, rc: ResourceCount, when: datetime):
-        if self.state == ReservationStates.Nascent or self.state == ReservationStates.Active or \
-                self.state == ReservationStates.ActiveTicketed or self.state == ReservationStates.Ticketed:
-            c = self.count_with_time(when=when)
-            if c.type is not None:
-                rc.tally_active(resource_type=c.type, count=c.active)
-                rc.tally_pending(resource_type=c.type, count=c.pending)
-        elif self.state == ReservationStates.Closed or self.state == ReservationStates.CloseWait and\
-                self.resources is not None:
-            rc.tally_close(resource_type=self.resources.type, count=self.resources.units)
-        elif self.state == ReservationStates.Failed and self.resources is not None:
-            rc.tally_failed(resource_type=self.resources.type, count=self.resources.units)
-
     def extend_lease(self):
         # Not permitted if there is a pending operation.
         self.nothing_pending()
@@ -840,10 +812,6 @@ class ReservationClient(Reservation, ABCControllerReservation):
             result = result[:-2]
         return result
 
-    def is_active(self) -> bool:
-        return (self.state == ReservationStates.Active or self.state == ReservationStates.ActiveTicketed) and \
-               self.joinstate == JoinState.NoJoin
-
     def is_active_joined(self) -> bool:
         return self.is_active() and self.joinstate == JoinState.NoJoin
 
@@ -1168,6 +1136,7 @@ class ReservationClient(Reservation, ABCControllerReservation):
     def service_update_ticket(self):
         if self.last_ticket_update.successful():
             self.resources.service_update(reservation=self)
+        if self.resources is not None and self.resources.sliver is not None:
             self.update_slice_graph(sliver=self.resources.sliver)
 
     def set_broker(self, *, broker: ABCBrokerProxy) -> bool:
@@ -1648,9 +1617,12 @@ class ReservationClient(Reservation, ABCControllerReservation):
         elif self.state == ReservationStates.Failed:
             self.logger.warning("Reservation #{} has failed".format(self.get_reservation_id()))
 
-    def fail(self, *, message: str, exception: Exception = None):
+    def fail(self, *, message: str, exception: Exception = None, sliver: BaseSliver = None):
         super().fail(message=message, exception=exception)
-        if self.requested_resources is not None and self.requested_resources.sliver is not None:
+        if sliver is None and self.requested_resources is not None and self.requested_resources.sliver is not None:
+            sliver = self.requested_resources.sliver
+
+        if sliver is not None:
             self.update_slice_graph(sliver=self.requested_resources.sliver)
 
     def update_slice_graph(self, *, sliver: BaseSliver):
@@ -1659,18 +1631,33 @@ class ReservationClient(Reservation, ABCControllerReservation):
         :param sliver: sliver
         :return:
         """
-        self.logger.debug(f"Updating ASM for  Reservation# {self.rid} State# {self.get_reservation_state()} "
-                          f"Slice Graph# {self.slice.get_graph_id()}")
-        error_message = self.get_error_message()
-        if error_message is None:
-            error_message = self.get_last_ticket_update()
-        if error_message is None:
-            error_message = self.get_last_lease_update()
-        self.slice.update_slice_graph(sliver=sliver, rid=str(self.rid),
-                                      reservation_state=self.state.name,
-                                      error_message=error_message)
-        self.logger.debug(f"Update ASM completed for  Reservation# {self.rid} State# {self.get_reservation_state()} "
-                          f"Slice Graph# {self.slice.get_graph_id()}")
+        begin = time.time()
+        try:
+            self.logger.debug(f"Updating ASM for  Reservation# {self.rid} State# {self.get_reservation_state()} "
+                              f"Slice Graph# {self.slice.get_graph_id()}")
+            error_message = self.get_error_message()
+            if error_message is None:
+                error_message = self.get_last_ticket_update()
+            if error_message is None:
+                error_message = self.get_last_lease_update()
+            '''
+            self.slice.update_slice_graph(sliver=sliver, rid=str(self.rid),
+                                          reservation_state=self.state.name,
+                                          error_message=error_message)
+            '''
+            asm_thread = self.actor.get_asm_thread()
+            if asm_thread is not None:
+                asm_thread.enqueue(graph_id=self.slice.get_graph_id(),
+                                   sliver=sliver, rid=str(self.rid),
+                                   reservation_state=self.state.name,
+                                   error_message=error_message)
+            self.logger.debug(f"Update ASM completed for  Reservation# {self.rid} State# {self.get_reservation_state()} "
+                              f"Slice Graph# {self.slice.get_graph_id()}")
+
+        except Exception as e:
+            self.logger.error(f"Failed to update the ASM Graph: {e}")
+            self.logger.error(traceback.format_exc())
+        self.logger.info(f"ASM TIME: {time.time() - begin:.0f}")
 
     def mark_close_by_ticket_review(self, *, update_data: UpdateData):
         if self.last_ticket_update is not None:
