@@ -41,40 +41,87 @@ class SliceState(Enum):
     StableOK = enum.auto()
     Closing = enum.auto()
     Dead = enum.auto()
+    Modifying = enum.auto()
+    ModifyError = enum.auto()
+    ModifyOK = enum.auto()
+    All = enum.auto()   # used only for querying
 
     def __str__(self):
         return self.name
 
-    @staticmethod
-    def state_from_str(state: str):
-        if state is None:
-            return state
+    @classmethod
+    def list_values(cls) -> List[int]:
+        return list(map(lambda c: c.value, cls))
 
-        for t in SliceState:
-            if state == str(t):
-                return t
-
-        return None
+    @classmethod
+    def list_names(cls) -> List[str]:
+        return list(map(lambda c: c.name, cls))
 
     @staticmethod
-    def str_list_to_state_list(states: List[str]) -> list or None:
+    def list_values_ex_closing_dead() -> List[int]:
+        result = SliceState.list_values()
+        result.remove(SliceState.Closing.value)
+        result.remove(SliceState.Dead.value)
+        return result
+
+    @staticmethod
+    def translate_list(states: List[str]) -> List[int] or None:
         if states is None or len(states) == 0:
             return states
 
-        result = [SliceState.StableOK, SliceState.StableError, SliceState.Dead,
-                  SliceState.Closing, SliceState.Configuring, SliceState.Nascent]
-        states_to_exclude = []
-        for s in result:
-            if str(s) not in states:
-                states_to_exclude.append(s)
+        incoming_states = list(map(lambda x: x.lower(), states))
 
-        for s in states_to_exclude:
-            result.remove(s)
+        result = SliceState.list_values()
 
-        ret_val = []
-        for x in result:
-            ret_val.append(x.value)
-        return ret_val
+        if len(incoming_states) == 1 and incoming_states[0] == SliceState.All.name.lower():
+            return result
+
+        for s in SliceState:
+            if s.name.lower() not in incoming_states:
+                result.remove(s.value)
+
+        return result
+
+    @staticmethod
+    def translate(state_name: str):
+        if state_name.lower() == SliceState.Nascent.name.lower():
+            return SliceState.Nascent
+        elif state_name.lower() == SliceState.Configuring.name.lower():
+            return SliceState.Configuring
+        elif state_name.lower() == SliceState.StableOK.name.lower():
+            return SliceState.StableOK
+        elif state_name.lower() == SliceState.StableError.name.lower():
+            return SliceState.StableError
+        elif state_name.lower() == SliceState.ModifyOK.name.lower():
+            return SliceState.ModifyOK
+        elif state_name.lower() == SliceState.ModifyError.name.lower():
+            return SliceState.ModifyError
+        elif state_name.lower() == SliceState.Modifying.name.lower():
+            return SliceState.Modifying
+        elif state_name.lower() == SliceState.Closing.name.lower():
+            return SliceState.Closing
+        elif state_name.lower() == SliceState.Dead.name.lower():
+            return SliceState.Dead
+        else:
+            return SliceState.All
+
+    @staticmethod
+    def is_dead_or_closing(*, state) -> bool:
+        if state == SliceState.Dead or state == SliceState.Closing:
+            return True
+        return False
+
+    @staticmethod
+    def is_stable(*, state) -> bool:
+        if state == SliceState.StableOK or state == SliceState.StableError:
+            return True
+        return False
+
+    @staticmethod
+    def is_modified(*, state) -> bool:
+        if state == SliceState.ModifyOK or state == SliceState.ModifyError:
+            return True
+        return False
 
 
 class SliceCommand(Enum):
@@ -82,6 +129,7 @@ class SliceCommand(Enum):
     Modify = enum.auto()
     Delete = enum.auto()
     Reevaluate = enum.auto()
+    ModifyAccept = enum.auto()
 
 
 class SliceOperation:
@@ -134,11 +182,16 @@ class SliceStateMachine:
 
     MODIFY = SliceOperation(SliceCommand.Modify, SliceState.StableOK, SliceState.StableError, SliceState.Configuring)
 
+    MODIFY_ACCEPT = SliceOperation(SliceCommand.ModifyAccept, SliceState.ModifyOK, SliceState.ModifyError,
+                                   SliceState.Modifying)
+
     DELETE = SliceOperation(SliceCommand.Delete, SliceState.Nascent, SliceState.StableOK, SliceState.StableError,
-                            SliceState.Configuring, SliceState.Dead)
+                            SliceState.Configuring, SliceState.Modifying, SliceState.ModifyOK, SliceState.ModifyError,
+                            SliceState.Dead)
 
     REEVALUATE = SliceOperation(SliceCommand.Reevaluate, SliceState.Nascent, SliceState.StableOK,
-                                SliceState.StableError, SliceState.Configuring, SliceState.Dead, SliceState.Closing)
+                                SliceState.StableError, SliceState.Configuring, SliceState.Dead, SliceState.Closing,
+                                SliceState.Modifying, SliceState.ModifyError, SliceState.ModifyOK)
 
     def __init__(self, *, slice_id: ID):
         self.slice_guid = slice_id
@@ -178,7 +231,13 @@ class SliceStateMachine:
             self.state = SliceState.Configuring
 
         elif operation.command == SliceCommand.Modify:
-            self.state = SliceState.Configuring
+            self.state = SliceState.Modifying
+
+        elif operation.command == SliceCommand.ModifyAccept:
+            if self.state == SliceState.ModifyError:
+                self.state = SliceState.StableError
+            elif self.state == SliceState.ModifyOK:
+                self.state = SliceState.StableOK
 
         elif operation.command == SliceCommand.Delete:
             if self.state != SliceState.Dead:
@@ -205,7 +264,21 @@ class SliceStateMachine:
                                                  ReservationStates.Failed):
                     self.state = SliceState.Closing
 
-            elif self.state == SliceState.StableError or self.state == SliceState.StableOK:
+            elif self.state == SliceState.Modifying:
+                if not bins.has_state_other_than(ReservationStates.Active, ReservationStates.Closed):
+                    self.state = SliceState.ModifyOK
+
+                if (not bins.has_state_other_than(ReservationStates.Active, ReservationStates.Failed,
+                                                  ReservationStates.Closed)) and \
+                        bins.has_state(s=ReservationStates.Failed):
+                    self.state = SliceState.ModifyError
+
+                if not bins.has_state_other_than(ReservationStates.Closed, ReservationStates.CloseWait,
+                                                 ReservationStates.Failed):
+                    self.state = SliceState.Closing
+
+            elif self.state == SliceState.StableError or self.state == SliceState.StableOK or \
+                    self.state == SliceState.ModifyError or self.state == SliceState.ModifyOK:
                 if not bins.has_state_other_than(ReservationStates.Closed, ReservationStates.CloseWait,
                                                  ReservationStates.Failed):
                     self.state = SliceState.Dead
