@@ -107,8 +107,9 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             self.logger = actor.get_logger()
             self.id = actor.get_guid()
 
-    def get_slices(self, *, slice_id: ID, caller: AuthToken, slice_name: str = None,
-                   email: str = None, state: List[int] = None, project: str = None) -> ResultSliceAvro:
+    def get_slices(self, *, slice_id: ID, caller: AuthToken, slice_name: str = None, email: str = None,
+                   state: List[int] = None, project: str = None, limit: int = None,
+                   offset: int = None) -> ResultSliceAvro:
         result = ResultSliceAvro()
         result.status = ResultAvro()
 
@@ -121,7 +122,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             try:
                 try:
                     slice_list = self.db.get_slices(slice_id=slice_id, slice_name=slice_name, email=email,
-                                                    state=state, project_id=project)
+                                                    state=state, project_id=project, limit=limit, offset=offset)
                 except Exception as e:
                     self.logger.error("getSlices:db access {}".format(e))
                     result.status.set_code(ErrorCodes.ErrorDatabaseError.value)
@@ -152,10 +153,8 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
 
                 slice_obj_new = SliceFactory.create(slice_id=ID(), name=slice_obj.get_slice_name())
                 slice_obj_new.set_description(description=slice_obj.get_description())
-                slice_obj_new.set_owner(owner=self.actor.get_identity())
-                slice_obj_new.get_owner().set_oidc_sub_claim(oidc_sub_claim=slice_obj.get_owner().get_oidc_sub_claim())
-                slice_obj_new.get_owner().set_email(email=slice_obj.get_owner().get_email())
-                slice_obj_new.get_owner().set_token(token=slice_obj.get_owner().get_token())
+                auth_token = Translate.translate_auth_from_avro(auth_avro=slice_obj.get_owner())
+                slice_obj_new.set_owner(owner=auth_token)
                 slice_obj_new.set_graph_id(graph_id=slice_obj.graph_id)
                 slice_obj_new.set_config_properties(value=slice_obj.get_config_properties())
                 slice_obj_new.set_lease_end(lease_end=slice_obj.get_lease_end())
@@ -195,6 +194,31 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
 
         return result
 
+    def accept_update_slice(self, *, slice_id: ID, caller: AuthToken) -> ResultAvro:
+        result = ResultAvro()
+        if slice_id is None or caller is None:
+            result.set_code(ErrorCodes.ErrorInvalidArguments.value)
+            result.set_message(ErrorCodes.ErrorInvalidArguments.interpret())
+            return result
+
+        try:
+
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCActorMixin):
+                    self.actor = actor
+
+                def run(self):
+                    self.actor.modify_accept(slice_id=slice_id)
+                    return None
+
+            self.actor.execute_on_actor_thread_and_wait(runnable=Runner(actor=self.actor))
+        except Exception as e:
+            self.logger.error("accept_update_slice: {}".format(e))
+            result.set_code(ErrorCodes.ErrorInternalError.value)
+            result.set_message(ErrorCodes.ErrorInternalError.interpret(exception=e))
+            result = ManagementObject.set_exception_details(result=result, e=e)
+        return result
+
     def remove_slice(self, *, slice_id: ID, caller: AuthToken) -> ResultAvro:
         result = ResultAvro()
         if slice_id is None or caller is None:
@@ -220,7 +244,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             result = ManagementObject.set_exception_details(result=result, e=e)
         return result
 
-    def update_slice(self, *, slice_mng: SliceAvro, caller: AuthToken) -> ResultAvro:
+    def update_slice(self, *, slice_mng: SliceAvro, caller: AuthToken, modify_state: bool = False) -> ResultAvro:
         result = ResultAvro()
         if slice_mng is None or slice_mng.get_slice_id() is None or caller is None:
             result.set_code(ErrorCodes.ErrorInvalidArguments.value)
@@ -235,24 +259,29 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                     self.actor = actor
 
                 def run(self):
-                    result = ResultAvro()
-                    slice_obj = self.actor.get_slice(slice_id=slice_id)
-                    if slice_obj is None:
-                        result.set_code(ErrorCodes.ErrorNoSuchSlice.value)
-                        result.set_message(ErrorCodes.ErrorNoSuchSlice.interpret())
-                        return result
-                    slice_obj = ManagementUtils.update_slice(slice_obj=slice_obj, slice_mng=slice_mng)
-                    slice_obj.set_dirty()
-
+                    runner_result = ResultAvro()
                     try:
-                        self.actor.get_plugin().get_database().update_slice(slice_object=slice_obj)
-                    except Exception as e:
-                        self.actor.get_logger().error("Could not commit slice update {}".format(e))
-                        result.set_code(ErrorCodes.ErrorDatabaseError.value)
-                        result.set_message(ErrorCodes.ErrorDatabaseError.interpret(exception=e))
-                        result = ManagementObject.set_exception_details(result=result, e=e)
+                        if modify_state:
+                            slice_object = Translate.translate_slice(slice_avro=slice_mng)
+                            self.actor.modify_slice(slice_object=slice_object)
+                        else:
+                            slice_obj = self.actor.get_slice(slice_id=slice_id)
+                            if slice_obj is None:
+                                runner_result.set_code(ErrorCodes.ErrorNoSuchSlice.value)
+                                runner_result.set_message(ErrorCodes.ErrorNoSuchSlice.interpret())
+                                return runner_result
 
-                    return result
+                            slice_obj = ManagementUtils.update_slice(slice_obj=slice_obj, slice_mng=slice_mng)
+                            slice_obj.set_dirty()
+
+                            self.actor.get_plugin().get_database().update_slice(slice_object=slice_obj)
+                    except Exception as e:
+                        self.actor.get_logger().error("Failed to modify/update slice {}".format(e))
+                        runner_result.set_code(ErrorCodes.ErrorDatabaseError.value)
+                        runner_result.set_message(ErrorCodes.ErrorDatabaseError.interpret(exception=e))
+                        runner_result = ManagementObject.set_exception_details(result=result, e=e)
+
+                    return runner_result
 
             result = self.actor.execute_on_actor_thread_and_wait(runnable=Runner(actor=self.actor))
         except Exception as e:
@@ -393,7 +422,7 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
                     self.actor.close_slice_reservations(slice_id=slice_id)
                     return None
 
-            self.actor.execute_on_actor_thread_and_wait(runnable=Runner(actor=self.actor))
+            self.actor.execute_on_actor_thread(runnable=Runner(actor=self.actor))
         except SliceNotFoundException as e:
             self.logger.error("close_slice_reservations: {}".format(e))
             result.set_code(ErrorCodes.ErrorNoSuchSlice.value)
@@ -563,3 +592,64 @@ class ActorManagementObject(ManagementObject, ABCActorManagementObject):
             GlobalsSingleton.get().create_maintenance_lock()
         else:
             GlobalsSingleton.get().delete_maintenance_lock()
+
+    def close_delegation(self, *, caller: AuthToken, did: str) -> ResultAvro:
+        result = ResultAvro()
+
+        if did is None or caller is None:
+            result.set_code(ErrorCodes.ErrorInvalidArguments.value)
+            result.set_message(ErrorCodes.ErrorInvalidArguments.interpret())
+            return result
+
+        try:
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCActorMixin):
+                    self.actor = actor
+
+                def run(self):
+                    self.actor.close_delegation(did=did)
+                    return True
+
+            self.actor.execute_on_actor_thread_and_wait(runnable=Runner(actor=self.actor))
+        except ReservationNotFoundException as e:
+            self.logger.error("close_reservation: {}".format(e))
+            result.set_code(ErrorCodes.ErrorNoSuchReservation.value)
+            result.set_message(e.text)
+        except Exception as e:
+            self.logger.error("close_reservation: {}".format(e))
+            result.set_code(ErrorCodes.ErrorInternalError.value)
+            result.set_message(ErrorCodes.ErrorInternalError.interpret(exception=e))
+            result = ManagementObject.set_exception_details(result=result, e=e)
+
+        return result
+
+    def remove_delegation(self, *, caller: AuthToken, did: str) -> ResultAvro:
+        result = ResultAvro()
+
+        if did is None or caller is None:
+            result.set_code(ErrorCodes.ErrorInvalidArguments.value)
+            result.set_message(ErrorCodes.ErrorInvalidArguments.interpret())
+            return result
+
+        try:
+
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCActorMixin):
+                    self.actor = actor
+
+                def run(self):
+                    self.actor.remove_delegation(did=did)
+                    return None
+
+            self.actor.execute_on_actor_thread_and_wait(runnable=Runner(actor=self.actor))
+        except ReservationNotFoundException as e:
+            self.logger.error("remove_delegation: {}".format(e))
+            result.set_code(ErrorCodes.ErrorNoSuchDelegation.value)
+            result.set_message(e.text)
+        except Exception as e:
+            self.logger.error("remove_delegation: {}".format(e))
+            result.set_code(ErrorCodes.ErrorInternalError.value)
+            result.set_message(ErrorCodes.ErrorInternalError.interpret(exception=e))
+            result = ManagementObject.set_exception_details(result=result, e=e)
+
+        return result
