@@ -27,13 +27,12 @@ from __future__ import annotations
 
 import traceback
 from datetime import datetime
-from typing import TYPE_CHECKING, List, Tuple
+from typing import TYPE_CHECKING, List
 
 from fabric_mb.message_bus.messages.lease_reservation_avro import LeaseReservationAvro
 from fabric_mb.message_bus.messages.result_delegation_avro import ResultDelegationAvro
 from fabric_mb.message_bus.messages.result_broker_query_model_avro import ResultBrokerQueryModelAvro
 from fabric_mb.message_bus.messages.result_proxy_avro import ResultProxyAvro
-from fabric_mb.message_bus.messages.result_reservation_avro import ResultReservationAvro
 from fabric_mb.message_bus.messages.result_string_avro import ResultStringAvro
 from fabric_mb.message_bus.messages.result_strings_avro import ResultStringsAvro
 from fabric_mb.message_bus.messages.result_avro import ResultAvro
@@ -52,14 +51,10 @@ from fabric_cf.actor.core.manage.management_object import ManagementObject
 from fabric_cf.actor.core.manage.management_utils import ManagementUtils
 from fabric_cf.actor.core.proxies.kafka.translate import Translate
 from fabric_cf.actor.core.time.actor_clock import ActorClock
-from fabric_cf.actor.security.access_checker import AccessChecker
-from fabric_cf.actor.security.pdp_auth import ActionId
 from fabric_cf.actor.core.apis.abc_client_actor_management_object import ABCClientActorManagementObject
 from fabric_cf.actor.core.time.term import Term
 from fabric_cf.actor.core.util.id import ID
-from fabric_cf.actor.core.util.resource_type import ResourceType
 from fabric_cf.actor.core.core.broker_policy import BrokerPolicy
-from fabric_cf.actor.security.pdp_auth import ResourceType as AuthResourceType
 
 if TYPE_CHECKING:
     from fabric_mb.message_bus.messages.proxy_avro import ProxyAvro
@@ -225,12 +220,10 @@ class ClientActorManagementObjectHelper(ABCClientActorManagementObject):
                 def run(self):
                     return self.parent.add_reservation_private(reservation=reservation)
 
-            rid, result.status = self.client.execute_on_actor_thread_and_wait(runnable=Runner(parent=self))
-
-            if rid is not None:
-                result.set_result(str(rid))
+            self.client.execute_on_actor_thread(runnable=Runner(parent=self))
         except Exception as e:
             self.logger.error("add_reservation {}".format(e))
+            self.logger.error(traceback.format_exc())
             result.status.set_code(ErrorCodes.ErrorInternalError.value)
             result.status.set_message(ErrorCodes.ErrorInternalError.interpret(exception=e))
             result.status = ManagementObject.set_exception_details(result=result.status, e=e)
@@ -372,8 +365,7 @@ class ClientActorManagementObjectHelper(ABCClientActorManagementObject):
 
                     return result
 
-            result = self.client.execute_on_actor_thread_and_wait(runnable=Runner(actor=self.client,
-                                                                                  logger=self.logger))
+            self.client.execute_on_actor_thread(runnable=Runner(actor=self.client, logger=self.logger))
         except Exception as e:
             self.logger.error("demand_reservation {}".format(e))
             result.set_code(ErrorCodes.ErrorInternalError.value)
@@ -382,11 +374,11 @@ class ClientActorManagementObjectHelper(ABCClientActorManagementObject):
 
         return result
 
-    def extend_reservation(self, *, reservation: id, new_end_time: datetime, new_units: int,
+    def extend_reservation(self, *, reservation: id, new_end_time: datetime, sliver: BaseSliver,
                            caller: AuthToken) -> ResultAvro:
         result = ResultAvro()
 
-        if reservation is None or caller is None or new_end_time is None:
+        if reservation is None or caller is None or (new_end_time is None and sliver is None):
             result.set_code(ErrorCodes.ErrorInvalidArguments.value)
             result.set_message(ErrorCodes.ErrorInvalidArguments.interpret())
             return result
@@ -405,19 +397,20 @@ class ClientActorManagementObjectHelper(ABCClientActorManagementObject):
                         return result
 
                     rset = ResourceSet()
-                    if new_units == Constants.EXTEND_SAME_UNITS:
-                        rset.set_units(units=r.get_resources().get_units())
-                    else:
-                        rset.set_units(units=new_units)
-
+                    units = r.get_resources().get_units()
+                    rset.set_units(units=units)
                     rset.set_type(rtype=r.get_resources().get_type())
 
-                    tmp_start_time = r.get_term().get_start_time()
-                    new_term = r.get_term().extend()
+                    new_term = r.get_term()
+                    if new_end_time is not None:
+                        tmp_start_time = r.get_term().get_start_time()
+                        new_term = r.get_term().extend()
 
-                    new_term.set_end_time(date=new_end_time)
-                    new_term.set_new_start_time(date=tmp_start_time)
-                    new_term.set_start_time(date=tmp_start_time)
+                        new_term.set_end_time(date=new_end_time)
+                        new_term.set_new_start_time(date=tmp_start_time)
+                        new_term.set_start_time(date=tmp_start_time)
+                    if sliver is not None:
+                        rset.set_sliver(sliver=sliver)
 
                     self.actor.extend(rid=r.get_reservation_id(), resources=rset, term=new_term)
 
