@@ -34,11 +34,11 @@ from fabric_mb.message_bus.messages.result_string_avro import ResultStringAvro
 from fabric_mb.message_bus.messages.result_avro import ResultAvro
 from fabric_mb.message_bus.messages.result_slice_avro import ResultSliceAvro
 
-from fabric_cf.actor.core.apis.i_actor_runnable import IActorRunnable
-from fabric_cf.actor.core.apis.i_reservation import ReservationCategory
+from fabric_cf.actor.core.apis.abc_actor_runnable import ABCActorRunnable
+from fabric_cf.actor.core.apis.abc_reservation_mixin import ReservationCategory
 from fabric_cf.actor.core.common.constants import Constants, ErrorCodes
 from fabric_cf.actor.core.kernel.slice import SliceTypes
-from fabric_cf.actor.core.kernel.slice_factory import SliceFactory
+from fabric_cf.actor.core.kernel.slice import SliceFactory
 from fabric_cf.actor.core.manage.actor_management_object import ActorManagementObject
 from fabric_cf.actor.core.manage.converter import Converter
 from fabric_cf.actor.core.manage.management_object import ManagementObject
@@ -48,15 +48,15 @@ from fabric_cf.actor.core.proxies.kafka.translate import Translate
 if TYPE_CHECKING:
     from fabric_mb.message_bus.messages.slice_avro import SliceAvro
 
-    from fabric_cf.actor.core.apis.i_actor import IActor
-    from fabric_cf.actor.core.apis.i_server_actor import IServerActor
+    from fabric_cf.actor.core.apis.abc_actor_mixin import ABCActorMixin
+    from fabric_cf.actor.core.apis.abc_server_actor import ABCServerActor
     from fabric_cf.actor.security.auth_token import AuthToken
     from fabric_cf.actor.core.util.id import ID
     from fabric_cf.actor.core.manage.messages.client_mng import ClientMng
 
 
 class ServerActorManagementObject(ActorManagementObject):
-    def __init__(self, *, sa: IServerActor = None):
+    def __init__(self, *, sa: ABCServerActor = None):
         super().__init__(actor=sa)
 
     def save(self) -> dict:
@@ -65,8 +65,8 @@ class ServerActorManagementObject(ActorManagementObject):
         properties[Constants.PROPERTY_MODULE_NAME] = ServerActorManagementObject.__module__
         return properties
 
-    def get_reservations_by_category(self, *, caller: AuthToken, category: ReservationCategory, slice_id: ID = None,
-                                     id_token: str = None) -> ResultReservationAvro:
+    def get_reservations_by_category(self, *, caller: AuthToken, category: ReservationCategory,
+                                     slice_id: ID = None) -> ResultReservationAvro:
         result = ResultReservationAvro()
         result.status = ResultAvro()
 
@@ -78,17 +78,11 @@ class ServerActorManagementObject(ActorManagementObject):
             res_list = None
             try:
                 if category == ReservationCategory.Client:
-                    if slice_id is None:
-                        res_list = self.db.get_client_reservations()
-                    else:
-                        res_list = self.db.get_client_reservations_by_slice_id(slice_id=slice_id)
+                    res_list = self.db.get_client_reservations(slice_id=slice_id)
                 elif category == ReservationCategory.Broker:
                     res_list = self.db.get_broker_reservations()
                 elif category == ReservationCategory.Inventory:
-                    if slice_id is None:
-                        res_list = self.db.get_holdings()
-                    else:
-                        res_list = self.db.get_holdings_by_slice_id(slice_id=slice_id)
+                    res_list = self.db.get_holdings(slice_id=slice_id)
             except Exception as e:
                 self.logger.error("do_get_reservations:db access {}".format(e))
                 result.status.set_code(ErrorCodes.ErrorDatabaseError.value)
@@ -111,8 +105,7 @@ class ServerActorManagementObject(ActorManagementObject):
 
         return result
 
-    def get_slices_by_slice_type(self, *, caller: AuthToken, slice_type: SliceTypes,
-                                 id_token: str = None) -> ResultSliceAvro:
+    def get_slices_by_slice_type(self, *, caller: AuthToken, slice_type: SliceTypes) -> ResultSliceAvro:
         result = ResultSliceAvro()
         result.status = ResultAvro()
 
@@ -126,9 +119,9 @@ class ServerActorManagementObject(ActorManagementObject):
 
             try:
                 if slice_type == SliceTypes.ClientSlice:
-                    slc_list = self.db.get_client_slices()
+                    slc_list = self.db.get_slices(slc_type=[SliceTypes.ClientSlice, SliceTypes.BrokerClientSlice])
                 elif slice_type == SliceTypes.InventorySlice:
-                    slc_list = self.db.get_inventory_slices()
+                    slc_list = self.db.get_slices(slc_type=[SliceTypes.InventorySlice])
 
             except Exception as e:
                 self.logger.error("get_slices_by_slice_type:db access {}".format(e))
@@ -179,8 +172,8 @@ class ServerActorManagementObject(ActorManagementObject):
             assert owner is not None
             slice_obj.set_owner(owner=owner)
 
-            class Runner(IActorRunnable):
-                def __init__(self, *, actor: IActor):
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCActorMixin):
                     self.actor = actor
 
                 def run(self):
@@ -216,8 +209,8 @@ class ServerActorManagementObject(ActorManagementObject):
             client_obj = Converter.fill_client(client_mng=client)
             client_obj.set_kafka_topic(kafka_topic=kafka_topic)
 
-            class Runner(IActorRunnable):
-                def __init__(self, *, actor: IActor):
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCServerActor):
                     self.actor = actor
 
                 def run(self):
@@ -234,7 +227,7 @@ class ServerActorManagementObject(ActorManagementObject):
 
         return result
 
-    def get_clients(self, *, caller: AuthToken, guid: ID = None, id_token: str = None) -> ResultClientMng:
+    def get_clients(self, *, caller: AuthToken, guid: ID = None) -> ResultClientMng:
         result = ResultClientMng()
         result.status = ResultAvro()
 
@@ -272,8 +265,8 @@ class ServerActorManagementObject(ActorManagementObject):
             return result
 
         try:
-            class Runner(IActorRunnable):
-                def __init__(self, *, actor: IActor):
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCServerActor):
                     self.actor = actor
 
                 def run(self):
@@ -301,8 +294,8 @@ class ServerActorManagementObject(ActorManagementObject):
             return result
 
         try:
-            class Runner(IActorRunnable):
-                def __init__(self, *, actor: IActor):
+            class Runner(ABCActorRunnable):
+                def __init__(self, *, actor: ABCServerActor):
                     self.actor = actor
 
                 def run(self):
