@@ -25,15 +25,20 @@
 # Author: Komal Thareja (kthare10@renci.org)
 import threading
 import traceback
-from datetime import datetime
+from datetime import datetime, timezone
+from typing import Tuple
+
 from fim.graph.resources.neo4j_arm import Neo4jARMGraph
 from fim.slivers.network_node import NodeSliver
+from fim.slivers.network_service import NetworkServiceSliver
 
 from fabric_cf.actor.core.apis.abc_authority_reservation import ABCAuthorityReservation
+from fabric_cf.actor.core.apis.abc_callback_proxy import ABCCallbackProxy
 from fabric_cf.actor.core.apis.abc_reservation_mixin import ABCReservationMixin
 from fabric_cf.actor.core.common.constants import Constants
 from fabric_cf.actor.core.core.authority_policy import AuthorityPolicy
 from fabric_cf.actor.core.common.exceptions import AuthorityException
+from fabric_cf.actor.core.kernel.reservation_states import ReservationStates
 from fabric_cf.actor.core.kernel.resource_set import ResourceSet
 from fabric_cf.actor.core.plugins.handlers.config_token import ConfigToken
 from fabric_cf.actor.core.apis.abc_resource_control import ABCResourceControl
@@ -403,13 +408,25 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
                 if node_id is None:
                     raise AuthorityException(f"Unable to find node_id {node_id} for reservation# {reservation}")
 
-                graph_node = self.get_node_from_graph(node_id=node_id)
+                graph_node = None
+                if isinstance(ticketed_sliver, NodeSliver):
+                    graph_node = self.get_network_node_from_graph(node_id=node_id)
+
+                elif isinstance(ticketed_sliver, NetworkServiceSliver):
+                    graph_node = self.get_network_service_from_graph(node_id=node_id)
+
+                else:
+                    msg = f'Reservation {reservation} sliver type is neither Node, nor NetworkServiceSliver'
+                    self.logger.error(msg)
+                    raise AuthorityException(msg)
+
                 self.logger.debug(f"Node {graph_node} serving reservation# {reservation}")
 
                 existing_reservations = self.get_existing_reservations(node_id=node_id,
                                                                        node_id_to_reservations=node_id_to_reservations)
 
-                delegation_name = self.get_delegation_name(delegation_id=requested.get_resources().get_delegation_id())
+                delegation_name, broker_callback = self.get_delegation_name_and_callback(
+                    delegation_id=requested.get_resources().get_delegation_id())
 
                 rset = rc.assign(reservation=reservation, delegation_name=delegation_name,
                                  graph_node=graph_node, existing_reservations=existing_reservations)
@@ -417,6 +434,7 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
                 if rset is None or rset.get_sliver() is None or rset.get_sliver().get_node_map() is None:
                     raise AuthorityException(f"Could not assign resources to reservation# {reservation}")
 
+                reservation.set_broker_callback(broker_callback=broker_callback)
                 return rset
             except Exception as e:
                 self.logger.error(traceback.format_exc())
@@ -433,7 +451,8 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
         else:
             raise AuthorityException(Constants.UNSUPPORTED_RESOURCE_TYPE.format(token.get_resource_type()))
 
-    def is_expired(self, *, reservation: ABCReservationMixin) -> bool:
+    @staticmethod
+    def is_expired(*, reservation: ABCReservationMixin) -> bool:
         """
         See if a reservation has expired
 
@@ -441,7 +460,7 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
         @return true if the reservation expired; otherwise, return false
         @raises Exception in case of error
         """
-        now = datetime.utcnow()
+        now = datetime.now(timezone.utc)
         end = reservation.get_term().get_end_time()
 
         return now > end
@@ -531,7 +550,7 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
                     break
             raise e
 
-    def get_node_from_graph(self, *, node_id: str) -> NodeSliver:
+    def get_network_node_from_graph(self, *, node_id: str) -> NodeSliver or None:
         try:
             self.lock.acquire()
             if self.aggregate_resource_model is None:
@@ -540,9 +559,23 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
         finally:
             self.lock.release()
 
+    def get_network_service_from_graph(self, *, node_id: str) -> NetworkServiceSliver or None:
+        try:
+            self.lock.acquire()
+            if self.aggregate_resource_model is None:
+                return None
+            return self.aggregate_resource_model.build_deep_ns_sliver(node_id=node_id)
+        finally:
+            self.lock.release()
+
     def get_existing_reservations(self, node_id: str, node_id_to_reservations: dict):
-        existing_reservations = self.actor.get_plugin().get_database().get_reservations_by_graph_node_id(
-            graph_node_id=node_id)
+        states = [ReservationStates.Active.value,
+                  ReservationStates.ActiveTicketed.value,
+                  ReservationStates.Ticketed.value,
+                  ReservationStates.Nascent.value]
+
+        existing_reservations = self.actor.get_plugin().get_database().get_reservations(graph_node_id=node_id,
+                                                                                        state=states)
 
         reservations_allocated_in_cycle = node_id_to_reservations.get(node_id, None)
 
@@ -561,12 +594,12 @@ class AuthorityCalendarPolicy(AuthorityPolicy):
 
         return existing_reservations
 
-    def get_delegation_name(self, *, delegation_id: str) -> str:
+    def get_delegation_name_and_callback(self, *, delegation_id: str) -> Tuple[str, ABCCallbackProxy]:
         try:
             self.lock.acquire()
             delegation = self.delegations.get(delegation_id, None)
-            if delegation is not None:
-                return delegation.get_delegation_name()
+            if delegation is not None and delegation.is_delegated():
+                return delegation.get_delegation_name(), delegation.get_callback()
         finally:
             self.lock.release()
 
