@@ -85,12 +85,13 @@ class Translate:
         if slice_avro.guid is None:
             raise ProxyException(Constants.NOT_SPECIFIED_PREFIX.format("Slice id"))
 
-        slice_obj = SliceFactory.create(slice_id=slice_avro.guid, name=slice_avro.slice_name)
+        slice_obj = SliceFactory.create(slice_id=ID(uid=slice_avro.guid), name=slice_avro.slice_name)
         slice_obj.set_description(description=slice_avro.description)
         slice_obj.set_config_properties(value=slice_avro.config_properties)
         slice_obj.set_lease_start(lease_start=slice_avro.get_lease_start())
         slice_obj.set_lease_end(lease_end=slice_avro.get_lease_end())
         slice_obj.set_project_id(project_id=slice_avro.project_id)
+        slice_obj.set_graph_id(graph_id=slice_avro.graph_id)
         return slice_obj
 
     @staticmethod
@@ -104,7 +105,7 @@ class Translate:
         avro_slice.config_properties = slice_obj.get_config_properties()
         avro_slice.set_lease_end(lease_end=slice_obj.get_lease_end())
         avro_slice.set_lease_start(lease_start=slice_obj.get_lease_start())
-        if slice_obj.__dict__.get('project_id') is not None:
+        if hasattr(slice_obj, 'project_id'):
             avro_slice.set_project_id(project_id=slice_obj.get_project_id())
 
         if slice_obj.get_resource_type() is not None:
@@ -124,8 +125,6 @@ class Translate:
         result.guid = str(auth.get_guid())
         result.oidc_sub_claim = auth.get_oidc_sub_claim()
         result.email = auth.get_email()
-        if AuthToken.__dict__.get('token') is not None:
-            result.token = auth.get_token()
         return result
 
     @staticmethod
@@ -137,7 +136,6 @@ class Translate:
         result.guid = auth_avro.guid
         result.oidc_sub_claim = auth_avro.oidc_sub_claim
         result.email = auth_avro.email
-        result.token = auth_avro.token
         return result
 
     @staticmethod
@@ -279,6 +277,7 @@ class Translate:
     @staticmethod
     def translate_delegation_to_avro(*, delegation: ABCDelegation) -> DelegationAvro:
         avro_delegation = DelegationAvro()
+        avro_delegation.delegation_name = delegation.get_delegation_name()
         avro_delegation.delegation_id = delegation.get_delegation_id()
         avro_delegation.state = delegation.get_state().value
         avro_delegation.slice = Translate.translate_slice_to_avro(slice_obj=delegation.get_slice_object())
