@@ -23,7 +23,9 @@
 #
 #
 # Author: Komal Thareja (kthare10@renci.org)
-from typing import Tuple, List
+import logging
+import threading
+from typing import Tuple, List, Union
 
 from fim.graph.abc_property_graph import ABCPropertyGraph, ABCGraphImporter
 from fim.graph.neo4j_property_graph import Neo4jGraphImporter, Neo4jPropertyGraph
@@ -42,7 +44,10 @@ from fim.slivers.delegations import Delegations
 from fim.slivers.interface_info import InterfaceSliver, InterfaceType
 from fim.slivers.network_node import NodeSliver
 from fim.slivers.network_service import NetworkServiceSliver
-from fim.user import ExperimentTopology, Labels, NodeType
+from fim.user import ExperimentTopology, Labels, NodeType, Component, ReservationInfo
+from fim.user.interface import Interface
+
+from fabric_cf.actor.core.kernel.reservation_states import ReservationStates
 
 
 class InterfaceSliverMapping:
@@ -107,14 +112,16 @@ class FimHelper:
     Provides methods to load Graph Models and perform various operations on them
     """
     @staticmethod
-    def get_neo4j_importer() -> ABCGraphImporter:
+    def get_neo4j_importer(neo4j_config: dict = None) -> ABCGraphImporter:
         """
         get fim graph importer
         :return: Neo4jGraphImporter
         """
-        from fabric_cf.actor.core.container.globals import GlobalsSingleton
-        neo4j_config = GlobalsSingleton.get().get_config().get_global_config().get_neo4j_config()
-        logger = GlobalsSingleton.get().get_logger()
+        logger = None
+        if neo4j_config is None:
+            from fabric_cf.actor.core.container.globals import GlobalsSingleton
+            neo4j_config = GlobalsSingleton.get().get_config().get_global_config().get_neo4j_config()
+            logger = GlobalsSingleton.get().get_logger()
 
         neo4j_graph_importer = Neo4jGraphImporter(url=neo4j_config["url"], user=neo4j_config["user"],
                                                   pswd=neo4j_config["pass"],
@@ -123,25 +130,27 @@ class FimHelper:
         return neo4j_graph_importer
 
     @staticmethod
-    def get_networkx_importer() -> ABCGraphImporter:
+    def get_networkx_importer(logger: logging.Logger = None) -> ABCGraphImporter:
         """
         get fim graph importer
         :return: Neo4jGraphImporter
         """
-        from fabric_cf.actor.core.container.globals import GlobalsSingleton
-        logger = GlobalsSingleton.get().get_logger()
+        if logger is None:
+            from fabric_cf.actor.core.container.globals import GlobalsSingleton
+            logger = GlobalsSingleton.get().get_logger()
 
         return NetworkXGraphImporter(logger=logger)
 
     @staticmethod
-    def get_arm_graph_from_file(*, filename: str, graph_id: str = None) -> ABCARMPropertyGraph:
+    def get_arm_graph_from_file(*, filename: str, graph_id: str = None, neo4j_config: dict = None) -> ABCARMPropertyGraph:
         """
         Load specified file directly with no manipulations or validation
         :param filename:
         :param graph_id:
+        :param neo4j_config neo4j_config
         :return:
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         neo4_graph = neo4j_graph_importer.import_graph_from_file(graph_file=filename, graph_id=graph_id)
         site_arm = Neo4jARMGraph(graph=Neo4jPropertyGraph(graph_id=neo4_graph.graph_id,
                                                           importer=neo4j_graph_importer))
@@ -151,13 +160,14 @@ class FimHelper:
         return site_arm
 
     @staticmethod
-    def get_arm_graph(*, graph_id: str) -> ABCARMPropertyGraph:
+    def get_arm_graph(*, graph_id: str, neo4j_config: dict = None) -> ABCARMPropertyGraph:
         """
         Load arm graph from fim
         :param graph_id: graph_id
+        :param neo4j_config neo4j_config
         :return: Neo4jARMGraph
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         arm_graph = Neo4jARMGraph(graph=Neo4jPropertyGraph(graph_id=graph_id, importer=neo4j_graph_importer))
         if arm_graph.graph_exists():
             arm_graph.validate_graph()
@@ -165,25 +175,39 @@ class FimHelper:
         return arm_graph
 
     @staticmethod
-    def get_graph(*, graph_id: str) -> ABCPropertyGraph:
+    def get_experiment_topology(*, graph_id: str) -> ExperimentTopology:
+        """
+        Load Experiment Topology provide Graph Id
+        :param graph_id Graph Id
+        """
+        graph = FimHelper.get_graph(graph_id=graph_id)
+        asm_graph = Neo4jASMFactory.create(graph=graph)
+        neo4j_topo = ExperimentTopology()
+        neo4j_topo.cast(asm_graph=asm_graph)
+        return neo4j_topo
+
+    @staticmethod
+    def get_graph(*, graph_id: str, neo4j_config: dict = None) -> ABCPropertyGraph:
         """
         Load arm graph from fim
-        :param graph_id: graph_id
+        :param graph_id: graph_id.
+        :param neo4j_config neo4j_config
         :return: Neo4jARMGraph
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         arm_graph = Neo4jPropertyGraph(graph_id=graph_id, importer=neo4j_graph_importer)
 
         return arm_graph
 
     @staticmethod
-    def get_neo4j_cbm_graph(graph_id: str) -> ABCCBMPropertyGraph:
+    def get_neo4j_cbm_graph(graph_id: str, neo4j_config: dict = None) -> ABCCBMPropertyGraph:
         """
         Load cbm graph from fim
         :param graph_id: graph_id
+        :param neo4j_config neo4j_config
         :return: Neo4jCBMGraph
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         combined_broker_model = Neo4jCBMGraph(graph_id=graph_id,
                                               importer=neo4j_graph_importer,
                                               logger=neo4j_graph_importer.log)
@@ -205,25 +229,27 @@ class FimHelper:
         return Neo4jCBMFactory.create(neo4_graph)
 
     @staticmethod
-    def get_graph_from_string_direct(*, graph_str: str) -> ABCPropertyGraph:
+    def get_graph_from_string_direct(*, graph_str: str, neo4j_config: dict = None) -> ABCPropertyGraph:
         """
         Load arm graph from fim
         :param graph_str: graph_str
+        :param neo4j_config neo4j_config
         :return: Neo4jPropertyGraph
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         graph = neo4j_graph_importer.import_graph_from_string_direct(graph_string=graph_str)
 
         return graph
 
     @staticmethod
-    def get_graph_from_string(*, graph_str: str) -> ABCPropertyGraph:
+    def get_graph_from_string(*, graph_str: str, neo4j_config: dict = None) -> ABCPropertyGraph:
         """
         Load arm graph from fim
         :param graph_str: graph_str
+        :param neo4j_config neo4j_config
         :return: Neo4jPropertyGraph
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         graph = neo4j_graph_importer.import_graph_from_string(graph_string=graph_str)
 
         return graph
@@ -241,12 +267,13 @@ class FimHelper:
         return graph
 
     @staticmethod
-    def delete_graph(*, graph_id: str):
+    def delete_graph(*, graph_id: str, neo4j_config: dict = None):
         """
         Delete a graph
         @param graph_id graph id
+        @param neo4j_config neo4j_config
         """
-        neo4j_graph_importer = FimHelper.get_neo4j_importer()
+        neo4j_graph_importer = FimHelper.get_neo4j_importer(neo4j_config=neo4j_config)
         neo4j_graph_importer.delete_graph(graph_id=graph_id)
 
     @staticmethod
@@ -281,22 +308,33 @@ class FimHelper:
                                     capacity_allocations=sliver.capacity_allocations,
                                     reservation_info=sliver.reservation_info,
                                     node_map=sliver.node_map,
-                                    management_ip=sliver.management_ip)
+                                    management_ip=sliver.management_ip,
+                                    capacity_hints=sliver.capacity_hints)
                 if sliver.attached_components_info is not None:
+                    graph_sliver = asm_graph.build_deep_node_sliver(node_id=sliver.node_id)
+                    diff = graph_sliver.diff(other_sliver=sliver)
+                    if diff is not None:
+                        for cname in diff.removed.components:
+                            reservation_info = ReservationInfo()
+                            reservation_info.reservation_state = ReservationStates.Failed.name
+                            node.components[cname].set_properties(reservation_info=reservation_info)
+
                     for component in sliver.attached_components_info.devices.values():
                         cname = component.get_name()
                         node.components[cname].set_properties(label_allocations=component.label_allocations,
                                                               capacity_allocations=component.capacity_allocations,
                                                               node_map=component.node_map)
                         # Update Mac address
-                        for ns in component.network_service_info.network_services.values():
-                            if ns.interface_info is None or ns.interface_info.interfaces is None:
-                                continue
+                        if component.network_service_info is not None and \
+                                component.network_service_info.network_services is not None:
+                            for ns in component.network_service_info.network_services.values():
+                                if ns.interface_info is None or ns.interface_info.interfaces is None:
+                                    continue
 
-                            for ifs in ns.interface_info.interfaces.values():
-                                topo_component = node.components[cname]
-                                topo_ifs = topo_component.interfaces[ifs.get_name()]
-                                topo_ifs.set_properties(label_allocations=ifs.label_allocations)
+                                for ifs in ns.interface_info.interfaces.values():
+                                    topo_component = node.components[cname]
+                                    topo_ifs = topo_component.interfaces[ifs.get_name()]
+                                    topo_ifs.set_properties(label_allocations=ifs.label_allocations)
 
             elif isinstance(sliver, NetworkServiceSliver):
                 node = neo4j_topo.network_services[node_name]
@@ -460,3 +498,32 @@ class FimHelper:
         switch = bqm.build_deep_node_sliver(node_id=sw_id)
 
         return switch, mpls_ns
+
+    @staticmethod
+    def get_parent_node(*, graph_model: ABCPropertyGraph, component: Component = None,
+                        interface: Interface = None) -> Tuple[Union[NodeSliver, NetworkServiceSliver, None], str]:
+        if component is not None:
+            node_name, node_id = graph_model.get_parent(node_id=component.node_id, rel=ABCPropertyGraph.REL_HAS,
+                                                        parent=ABCPropertyGraph.CLASS_NetworkNode)
+            node = graph_model.build_deep_node_sliver(node_id=node_id)
+        elif interface is not None:
+            node_name, node_id = graph_model.get_parent(node_id=interface.node_id, rel=ABCPropertyGraph.REL_CONNECTS,
+                                                        parent=ABCPropertyGraph.CLASS_NetworkService)
+            node = graph_model.build_deep_ns_sliver(node_id=node_id)
+        else:
+            raise Exception("Invalid Arguments - component/interface both are None")
+
+        return node, node_id
+
+    @staticmethod
+    def prune_graph(*, graph_id: str) -> ExperimentTopology:
+        """
+        Load arm graph from fim and prune all nodes with reservation_state = reservation_state
+        :param graph_id: graph_id
+        :return: ExperimentTopology
+        """
+        slice_topology = FimHelper.get_experiment_topology(graph_id=graph_id)
+        slice_topology.prune(reservation_state=ReservationStates.Failed.name)
+        slice_topology.prune(reservation_state=ReservationStates.Closed.name)
+
+        return slice_topology
