@@ -3,7 +3,10 @@ from typing import List
 controller generated to handled auth operation described at:
 https://connexion.readthedocs.io/en/latest/security.html
 """
+from fabric_cf.orchestrator.swagger_server.response import authorization_controller as rc
+
+
 def check_bearerAuth(token):
-    return {'test_key': 'test_value'}
+    return rc.check_bearerAuth(token)
 
 
